unit UTbmp;
interface
uses UTApli, Graph;
type
{
 * Informacin del archivo :

Tipo de archivo                InfoHeader.bfType
Tamao del archivo             InfoHeader.bfSize
Reservado 1                    InfoHeader.bfReserved1
Reservado 2                    InfoHeader.bfReserved2
Comienzo de los datos          InfoHeader.bfOffbits

 * Informacin del Bitmap :

Tamao de la estructura        InfoHeader.biSize
Ancho                          InfoHeader.biWidth
Alto                           InfoHeader.biHeight
Planos                         InfoHeader.biPlanes
Bits por pixel                 InfoHeader.biBitCount
Compresin                     InfoHeader.biCompression
Tamao de la imagen            InfoHeader.biSizeImage
Pixels por metro en X          InfoHeader.biXpelsPerMeter
Pixels por metro en Y          InfoHeader.biYpelsPerMeter
Colores utilizados             InfoHeader.biClrUsed
Colores importantes            InfoHeader.biClrImportant}

    BitMapHeader = record
                   {BitMap File Header}
                     bfType:integer;
                     bfSize:longint;
                     bfReserved1:integer;
                     bfReserved2:integer;
                     bfOffbits:longint;
                   {BitMap Info Header}
                     biSize:longint;
                     biWidth:longint;
                     biHeight:longint;
                     biPlanes:integer;
                     biBitCount:integer;
                     biCompression:longint;
                     biSizeImage:longint;
                     biXpelsPerMeter:longint;
                     biYpelsPerMeter:longint;
                     biClrUsed:longint;
                     biClrImportant:longint;
                    end;

{-------------------------[ Cabecera del fichero ]-----------------------}
        RGBQUAD = record
                    rgbBLUE, rgbGREEN, rgbRED, rgbReserved : byte;
                  end;

        Paleta  = array [0..255] of record
                                      R,G,B:Byte;
                                    end;

        ArrayBuf = array [1..64000] of byte;

        Tbmp = object (TObjeto)
                 constructor crear (x, y : word; s: string);
                private
                 MinX, MinY, MaxX, MaxY : word;
                 CorX1, CorY1, CorX2, CorY2 : word;
                 vCostado, vTope, vLargo, vAlto : word;
                 InfoHeader:BitMapHeader;
                 Colores: RGBQUAD;
                 NombreArchivo:string;
                 Arch : file;
                 Buffer : ^ArrayBuf;
                 Pal : Paleta;
                 procedure MostrarBmp256;
                 procedure MostrarBmp16;
               end;
var
  Pant: array [1..200,1..320] of byte absolute $A000:0000;

  procedure VGA320x200x256;
  procedure VGA640x480x16;

implementation
{----------------[ Rutinas de utilidad general ]-------------------}

 function Pot (A,B : real) : integer;
 begin
   Pot := round (exp (b * ln (a)));
 end;

{-----[ Rutinas para el modo gr fico VGA 320x200 256 colores ]-----}

 procedure VGA320x200x256;
 { Rutina para inicializar el modo VGA 320x200x256c }
 begin
   asm
    mov ax,0013h
    int 10h
   end;
 end;

 procedure Bar256 (X1,Y1,X2,Y2 : word; c : byte);
 var y : byte;
 begin
   for y := Y1 to Y2 do
      FillChar (Pant [Y,X1], X2-X1, C);
 end;

 procedure UsaPaleta (var P : paleta);
 {Nota: P.R, P.G, P.B deben ser 0<=.<=63}
 var i, j : byte;
 begin
   for i := 0 to 255 do
   begin
     Port[$3C8] := i;
     Port[$3C9] := P[i].R;
     Port[$3C9] := P[i].G;
     Port[$3C9] := P[i].B;
   end;
 end;

{------[ Rutinas para el modo grfico VGA 640x480 16 colores ]------}

 procedure VGA640x480x16;
 var Driver, Modo : integer;
 begin
   Driver := VGA;
   Modo := VGAHi;
   InitGraph(Driver, Modo, 'c:\instala\bgi');
   SetTextJustify(CenterText,CenterText);
 end;

 procedure UsaPaleta16 (var P : Paleta);
 var i:integer;
 begin
   for i := 0 to 15 do
   begin
     SetRGBPalette(i, P[i].R, P[i].G, P[i].B);
     SetPalette(i, i);
   end;
 end;

{------------------[ Rutinas del objeto TBmp ]----------------------}
 constructor TBmp.crear (x, y : word; s: string);
 begin
   vCostado := x;
   vTope := y;
   NombreArchivo := s;
   if Pos ('.', NombreArchivo) = 0 then
    NombreArchivo := NombreArchivo + '.BMP';
   Assign (Arch, NombreArchivo);
   Reset (Arch, 1);
   {OPTIENE INFO DE LA CABECERA}
   BLockRead (Arch,InfoHeader,Sizeof(InfoHeader));
   {Comprueba si es realmente un archivo BMP "BM = 19778"}
   if InfoHeader.bfType = 19778 then
   begin
    {Calcula el nmero de colores utilizados en la imagen}
    if InfoHeader.biClrUsed = 0  then
     infoHeader.biClrUsed := Pot (2,InfoHeader.biBitCount);
    case InfoHeader.biClrUsed of
          256 : MostrarBmp256;
          16  : MostrarBmp16;
    end;
   end;
 end;

{--------------------[ vizulalizacin con 256 colores ]---------------}
 procedure TBmp.MostrarBmp256;
 var
   BytesPorLinea : word;
   offset : longint;
   Y, alto, ancho : longint;
 begin
   BytesPorLinea := 4*((InfoHeader.biWidth + 3) shr 2);
   {Nota: el numero de bytes por linea es multiplo de 4 --> palabras de 32 bits}
   Offset := InfoHeader.bfOffbits;
   if InfoHeader.Biheight > 200 then
    Offset := InfoHeader.bfOffbits +
             (InfoHeader.Biheight-200) * BytesPorLinea;
   Seek (Arch, Offset);  {Posiciona el puntero de lectura al inicio}
                {de la seccin de datos del bitmap que cabe en la pantalla}
   { VGA320x200x256;       {Establece el modo gr fico 320x200 con 256 colores}
   UsaPaleta (pal);
   Alto:= InfoHeader.biHeight;
   if alto>200 then
    alto:=200;
    ancho := bytesporlinea;
   if ancho > 320 then
    ancho:=320;

   for y := Alto downto 1 do
   begin
     {copia el archivo a la pantalla}
     blockread(Arch, pant[y,1], ancho);
     {salta el trozo que se sale  de la pantalla}
     seek(Arch, FilePos(Arch)+bytesporlinea-ancho);
   end;
 end;

{--------------------[ vizulalizacin con 16 colores ]---------------}
 procedure Tbmp.MostrarBmp16;
 var
     BytesPorLinea : word;
     offset : longint;
     alto,ancho:longint;
     N,Y : word;
     A,B : byte;
 begin
   bytesporlinea := (InfoHeader.bfSize - InfoHeader.bfOffBits) div
                     InfoHeader.biHeight;

   Offset := InfoHeader.bfOffbits;
   if InfoHeader.Biheight > 480 then
    Offset := InfoHeader.bfOffbits +
             (InfoHeader.Biheight - 480) * bytesPorlinea;
   {Situa el puntero de lectura en el inicio de la seccin}
   {de datos del bitmap que cabe en la pantalla}
   seek (Arch, Offset);
   {VGA640x480x16;     {Establece el modo gr fico 640x480 con 16 colores}
   UsaPaleta16 (Pal);
   alto := InfoHeader.biHeight;
   if alto > 480 then
     alto := 480;
   ancho := InfoHeader.biWidth;
   if ancho > 640 then
    ancho := 640;

  getmem(buffer, bytesporlinea);
  for y := alto downto 1 do
  begin
    blockread(Arch, Buffer^, bytesPorLinea);
    for N := 1 to ancho div 2 do
    begin
      A := Buffer^[N] shr 4;
      B := Buffer^[N] and $F;
      PutPixel (N*2,Y,A);
      PutPixel (N*2+1,Y,B);
    end;
   end;
  freemem(buffer, bytesporlinea);
 end;

end.